
/*
 * Copyright 1994  The XFree86 Project
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL 
 * DAVID WEXELBLAT BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF 
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE.
 * 
 * Hardware Cursor for Trident utilizing XAA Cursor code.
 * Written by Alan Hourihane <alanh@fairlite.demon.co.uk>
 * Modified for Chips and Technologies by David Bateman <dbateman@eng.uts.edu.au>
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* All drivers should typically include these */
#include "xf86.h"
#include "xf86_OSproc.h"

/* Everything using inb/outb, etc needs "compiler.h" */
#include "compiler.h"   

/* Drivers for PCI hardware need this */
#include "xf86PciInfo.h"

/* Drivers that need to access the PCI config space directly need this */
#include "xf86Pci.h"

#include "xf86Cursor.h"

/* Driver specific headers */
#include "ct_driver.h"


static void
CHIPSShowCursor(ScrnInfoPtr pScrn)
{
    CHIPSPtr cPtr = CHIPSPTR(pScrn);
    vgaHWPtr hwp = VGAHWPTR(pScrn);

    unsigned char tmp;

#ifdef DEBUG
    ErrorF("%d:CHIPSShowCursor(): MSS 0x%X\n", pScrn->scrnIndex, cPtr->readMSS( cPtr ) );
#endif

    CHIPSHiQVSync(pScrn);

    /* turn the cursor on */
    tmp = cPtr->readXR(cPtr, 0xA0);

    cPtr->writeXR(cPtr, 0xA0, (tmp & 0xF8) | 5);


    cPtr->HWCursorShown = TRUE;
}

static void
CHIPSHideCursor(ScrnInfoPtr pScrn)
{
    unsigned char tmp;

    CHIPSPtr cPtr = CHIPSPTR(pScrn);

#ifdef DEBUG
    ErrorF("%d:CHIPSHideCursor(): MSS 0x%X\n", pScrn->scrnIndex, cPtr->readMSS( cPtr ) );
#endif

    CHIPSHiQVSync(pScrn);

    /* turn the cursor off */
    tmp = cPtr->readXR(cPtr, 0xA0);
    cPtr->writeXR(cPtr, 0xA0, tmp & 0xF8);

    cPtr->HWCursorShown = FALSE;
}

static void
CHIPSSetCursorPosition(ScrnInfoPtr pScrn, int x, int y)
{
    CHIPSPtr cPtr = CHIPSPTR(pScrn);

#ifdef DEBUG
    ErrorF("%d:CHIPSSetCursorPosition(): MSS 0x%X\n", pScrn->scrnIndex, cPtr->readMSS( cPtr ) );
#endif

    CHIPSHiQVSync(pScrn);
    
    if (x < 0)
	    x = ~(x-1) | 0x8000;
    if (y < 0)
	    y = ~(y-1) | 0x8000;

    /* Program the cursor origin (offset into the cursor bitmap). */
    cPtr->writeXR(cPtr, 0xA4, x & 0xFF);
    cPtr->writeXR(cPtr, 0xA5, (x >> 8) & 0x87);
    cPtr->writeXR(cPtr, 0xA6, y & 0xFF);
    cPtr->writeXR(cPtr, 0xA7, (y >> 8) & 0x87);
}

static void
CHIPSSetCursorColors(ScrnInfoPtr pScrn, int bg, int fg)
{
    CHIPSPtr cPtr = CHIPSPTR(pScrn);
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    CARD32 packedcolfg, packedcolbg;
  
#ifdef DEBUG
    ErrorF("%d:CHIPSSetCursorColors(): MSS 0x%X\n", pScrn->scrnIndex, cPtr->readMSS( cPtr ) );
#endif

    CHIPSHiQVSync(pScrn);

    unsigned char xr80;

    /* Enable extended palette addressing */
    xr80 = cPtr->readXR(cPtr, 0x80);

    cPtr->writeXR(cPtr, 0x80, xr80 | 0x1);

    /* Write the new colours to the extended VGA palette. Palette
     * index is incremented after each write, so only write index
     * once 
     */
    hwp->writeDacWriteAddr(hwp, 0x04);

    if (xr80 & 0x80) 
    {
        /* 8bit DAC */
        hwp->writeDacData(hwp, (bg >> 16) & 0xFF);
        hwp->writeDacData(hwp, (bg >> 8) & 0xFF);
        hwp->writeDacData(hwp, bg & 0xFF);
        hwp->writeDacData(hwp, (fg >> 16) & 0xFF);
        hwp->writeDacData(hwp, (fg >> 8) & 0xFF);
        hwp->writeDacData(hwp, fg & 0xFF);
    }         
    else 
    {
        /* 6bit DAC */
        hwp->writeDacData(hwp, (bg >> 18) & 0xFF);
        hwp->writeDacData(hwp, (bg >> 10) & 0xFF);
        hwp->writeDacData(hwp, (bg >> 2) & 0xFF);
        hwp->writeDacData(hwp, (fg >> 18) & 0xFF);
        hwp->writeDacData(hwp, (fg >> 10) & 0xFF);
        hwp->writeDacData(hwp, (fg >> 2) & 0xFF);
    }

    /* Enable normal palette addressing */
    cPtr->writeXR(cPtr, 0x80, xr80);
        
}

static void
CHIPSLoadCursorImage(ScrnInfoPtr pScrn, unsigned char *src)
{
    CHIPSPtr cPtr = CHIPSPTR(pScrn);
    vgaHWPtr hwp = VGAHWPTR(pScrn);
    CHIPSACLPtr cAcl = CHIPSACLPTR(pScrn);

#ifdef DEBUG
    ErrorF("%d:CHIPSLoadCursorImage(): MSS 0x%X\n", pScrn->scrnIndex, cPtr->readMSS( cPtr ) );
#endif

    CHIPSHiQVSync(pScrn);

    if ( cPtr->cursorDelay ) 
    {
	    usleep(200000);
	    cPtr->cursorDelay = FALSE;
    }
    
    /*
        There is no need for byte swapping of data for the cursor. 
        This is true since cPtr->FbBase was mapped to the frame
        buffer that is big Endian. (drk)

    */
    memcpy((unsigned char *)cPtr->FbBase + cAcl->CursorAddress,
	        src, cPtr->CursorInfoRec->MaxWidth * 
	        cPtr->CursorInfoRec->MaxHeight / 4);

    /* set cursor address here or we loose the cursor on video mode change */
    cPtr->writeXR(cPtr, 0xA2, ((cAcl->CursorAddress + cAcl->FbOffset ) >> 8) & 0xFF);
    cPtr->writeXR(cPtr, 0xA3, ((cAcl->CursorAddress + cAcl->FbOffset ) >> 16) & 0x3F);

}

static Bool
CHIPSUseHWCursor(ScreenPtr pScreen, CursorPtr pCurs)
{
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];

    CHIPSPtr cPtr = CHIPSPTR(pScrn);

#ifdef DEBUG
    ErrorF("%d:CHIPSUseHWCursor(): MSS 0x%X\n", pScrn->scrnIndex, cPtr->readMSS( cPtr ) );
#endif
    
    return (((cPtr->Flags & ChipsHWCursor) != 0)
	    && !(pScrn->currentMode->Flags & V_DBLSCAN));
}

Bool
CHIPSCursorInit(ScreenPtr pScreen)
{
    xf86CursorInfoPtr infoPtr;
    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
    CHIPSPtr cPtr = CHIPSPTR(pScrn);

#ifdef DEBUG
    ErrorF("%d:CHIPSCursorInit(): MSS 0x%X\n", pScrn->scrnIndex, cPtr->readMSS( cPtr ) );
#endif

    infoPtr = xf86CreateCursorInfoRec();

    if( !infoPtr ) 
        return FALSE;

    cPtr->CursorInfoRec = infoPtr;

    infoPtr->Flags =

/*#if X_BYTE_ORDER == X_LITTLE_ENDIAN*/

#if 0
	    HARDWARE_CURSOR_BIT_ORDER_MSBFIRST |
#endif
	    HARDWARE_CURSOR_INVERT_MASK |
	    HARDWARE_CURSOR_SWAP_SOURCE_AND_MASK |
	    HARDWARE_CURSOR_TRUECOLOR_AT_8BPP;

    infoPtr->Flags |= HARDWARE_CURSOR_SOURCE_MASK_INTERLEAVE_64;
    infoPtr->MaxHeight = 64;
    infoPtr->MaxWidth = 64;

    infoPtr->SetCursorColors = CHIPSSetCursorColors;
    infoPtr->SetCursorPosition = CHIPSSetCursorPosition;
    infoPtr->LoadCursorImage = CHIPSLoadCursorImage;
    infoPtr->HideCursor = CHIPSHideCursor;
    infoPtr->ShowCursor = CHIPSShowCursor;
    infoPtr->UseHWCursor = CHIPSUseHWCursor;

    return(xf86InitCursor(pScreen, infoPtr));
}

